@@ -201,6 +201,7 @@ XOpenDisplay (
 	X_DPY_SET_LAST_REQUEST_READ(dpy, 0);
 	dpy->default_screen = iscreen;  /* Value returned by ConnectDisplay */
 	dpy->last_req = (char *)&_dummy_request;
+	dpy->req_seq_syncing = False;
 
 	/* Initialize the display lock */
 	if (InitDisplayLock(dpy) != 0) {
@@ -332,7 +333,7 @@ XOpenDisplay (
 	}
 
  	u.setup = (xConnSetup *) (((char *) u.setup) + sz_xConnSetup);
-  	(void) strncpy(dpy->vendor, u.vendor, vendorlen);
+  	(void) strncpy(dpy->vendor, u.vendor, (size_t) vendorlen);
 	dpy->vendor[vendorlen] = '\0';
  	vendorlen = (vendorlen + 3) & ~3;	/* round up */
 	u.vendor += vendorlen;
